unit Data;

interface

uses
  SysUtils, Classes, My_Consts;

type
  TMeasure = (mt34, mt44, mt54);

  PTone = ^TTone;
  TTone = record
    Freq: integer;
    Length: integer;
  end;

type
  TData = class
  private
    FMeasure: TMeasure;
    FList: TList;
    function GetCount: integer;
  public
    constructor Create;
    destructor Destroy; override;

    property Count: integer read GetCount;
    property Measure: TMeasure read FMeasure write FMeasure;

    function GetTone(Index: integer): PTone;
    function LoadFromFile(AFileName: string): boolean;
  end;

implementation

constructor TData.Create;
begin
  inherited Create;

  FList := TList.Create;
end;

destructor TData.Destroy;
begin
  FList.Free;

  inherited Destroy;
end;

function TData.LoadFromFile(AFileName: string): boolean;

  procedure Parse(s: string; var s1: string; var s2: string);
  begin
    s1 := Copy(s, 1, LastDelimiter(' ', s) - 1);
    s2 := Copy(s, LastDelimiter(' ', s) + 1, MaxInt);
  end;

var
  f: TextFile;
  s, s1, s2: string;
  i: integer;
  AFreq, ALength: integer;
  Entry: PTone;

begin
  result := false;
  FList.Clear;

  try
    try
      AssignFile(f, AFileName);
      Reset(f);

      ReadLn(f, s);
      s := Trim(s);
      if s = SMeasure34 then
        FMeasure := mt34
      else
        if s = SMeasure44 then
          FMeasure := mt44
        else
          if s = SMeasure54 then
            FMeasure := mt54
          else
            Abort;

      while not EOF(f) do
      begin
        ReadLn(f, s);
        s := Trim(s);

        if s = '' then continue;
        Parse(s, s1, s2);
        if (s1 = '') or (s2 = '') then
          Abort;

        AFreq := -1;
        ALength := -1;

        for i := Low(SNoty) to High(SNoty) do
          if SNoty[i] = s1 then
          begin
            AFreq := i;
            break;
          end;

        for i := Low(SDelky) to High(SDelky) do
          if SDelky[i] = s2 then
          begin
            ALength := i;
            break;
          end;

        if (AFreq = -1) or (ALength = -1) then
          Abort;

        New(Entry);
        Entry^.Freq := AFreq;
        Entry^.Length := ALength;
        FList.Add(Entry);
      end;

    except
      FList.Clear;
      exit;
    end;

  finally
    {$I-}
    CloseFile(f);
    IOResult;
    {$I+}
  end;

  result := true;
end;

function TData.GetTone(Index: integer): PTone;
begin
  result := nil;
  if (Index >= 0) and (Index <= FList.Count - 1) then
    result := FList[Index];
end;

function TData.GetCount: integer;
begin
  result := FList.Count;
end;

end.
